﻿#include <stdio.h>
#include "ftd2xx.h"
#pragma comment(lib, "ftd2xx")

#define TX_SECTOR (64*1024)
#define RX_SECTOR (64*1024)

int myOpen(int portNumber,FT_HANDLE* ftHandle)
{
	FT_STATUS status = FT_Open(portNumber, ftHandle);
	if (status != FT_OK) 
	{
		// FT_Open can fail if the ftdi_sio module is already loaded.
		printf("FT_Open(%d) failed (error %d).\n", portNumber, (int)status);
		printf("Use lsmod to check if ftdi_sio (and usbserial) are present.\n");
		printf("If so, unload them using rmmod, as they conflict with ftd2xx.\n");
		return 0;
	}
	
	printf("FT_Open(%d) success.\n",portNumber);
	return 1;
}

void myClose(FT_HANDLE ftHandle)
{
	FT_STATUS status = FT_Close(ftHandle);
	if(status!=FT_OK){
		printf("FT_Close() failed (error %d).\n", (int)status);
		return;
	}

	printf("FT_Close() success.\n");
}

FT_STATUS mySetup(FT_HANDLE ftHandle)
{
	FT_STATUS status = FT_SetBitMode(ftHandle,0xFF,FT_BITMODE_RESET);// reset mode
	if(status!=FT_OK){
		printf("FT_SetBitMode(Reset) failed (error %d).\n", status);
		FT_Close(ftHandle);
		return status;
	}
	Sleep(1000);
	status = FT_SetBitMode(ftHandle,0xFF, FT_BITMODE_SYNC_FIFO);	// Sync FIFO mode
	if(status!=FT_OK){
		printf("FT_SetBitMode(Sync FIFO mode) failed (error %d)\n", status);
		FT_Close(ftHandle);
		return status;
	}

	UCHAR LatencyTimer = 2;
	status = FT_SetLatencyTimer(ftHandle,LatencyTimer);
	if(status!=FT_OK){
		printf("FT_SetLatencyTimer(%u) failed (error %d)\n",LatencyTimer, status);
		FT_Close(ftHandle);
		return status;
	}
	status = FT_SetUSBParameters(ftHandle,RX_SECTOR, TX_SECTOR); // (64KB, 64KB)
	if(status!=FT_OK){
		printf("FT_SetUSBParameters(%u,%u) failed (error %d)\n",RX_SECTOR,TX_SECTOR, status);
		FT_Close(ftHandle);
		return status;
	}
	
//	status = FT_SetBaudRate(ftHandle, 115200);
//	if(status!=FT_OK){
//		printf("FT_SetBaudRate(115200) failed (error %d)\n", status);
//		FT_Close(ftHandle);
//		return status;
//	}
//	status = FT_SetDtr(ftHandle);
//	if(status!=FT_OK){
//		printf("FT_SetDtr() failed (error %d)\n", status);
//		FT_Close(ftHandle);
//		return status;
//	}
//	status = FT_SetRts(ftHandle);
//	if(status!=FT_OK){
//		printf("FT_SetRts() failed (error %d)\n", status);
//		FT_Close(ftHandle);
//		return status;
//	}	
//	status = FT_SetDeadmanTimeout(ftHandle,10000);
//	if(status!=FT_OK){
//		printf("FT_SetDeadmanTimeout(10000) failed (error %d)\n", status);
//		FT_Close(ftHandle);
//		return status;
//	}

	ULONG timeoRx=1000,timeoTx=1000;
	status = FT_SetTimeouts(ftHandle,timeoRx,timeoTx);
	if(status!=FT_OK){
		printf("FT_SetTimeouts(%u,%u) failed (error %d)\n",timeoRx,timeoTx,status);
		FT_Close(ftHandle);
		return status;
	}
	status = FT_SetFlowControl(ftHandle,FT_FLOW_RTS_CTS,0,0);
	if(status!=FT_OK){
		printf("FT_SetFlowControl(%u,%u,%u) failed (error %d)\n",FT_FLOW_RTS_CTS,0,0,status);
		FT_Close(ftHandle);
		return status;
	}
	status = FT_Purge(ftHandle,FT_PURGE_TX|FT_PURGE_RX);
	if(status!=FT_OK){
		printf("Purge(FT_PURGE_TX|FT_PURGE_RX) failed (error %d)\n",status);
		FT_Close(ftHandle);
		return status;
	}
	
	printf("setup success.\n");
	return status;
}


int myWrite(FT_HANDLE ftHandle,LPVOID lpBuffer,DWORD dwBytesToWrite,LPDWORD lpBytesWritten)
{
	FT_STATUS status = FT_Write(ftHandle,lpBuffer,dwBytesToWrite,lpBytesWritten);
	if(status!=FT_OK){
		printf("FT_Write failed (error %d)\n",status);
		return 0;
	}else if(*lpBytesWritten<=0){
		printf("FT_Write failed [written %d]\n",*lpBytesWritten);
		return 0;
	}else{
		printf("FT_Write success. [written %d]\n",*lpBytesWritten);
		return 1;
	}
}

int myRead(FT_HANDLE ftHandle,LPVOID lpBuffer,DWORD dwBytesToRead,LPDWORD lpBytesRead)
{
	FT_STATUS status = FT_Read(ftHandle,lpBuffer,dwBytesToRead,lpBytesRead);
	if(status!=FT_OK){
		printf("FT_Read failed (error %d)\n",status);
		return 0;
	}else if(*lpBytesRead<=0){
		printf("FT_Read failed [read %d]\n",*lpBytesRead);
		return 0;
	}else{
		printf("FT_Read success. [read %d]\n",*lpBytesRead);
		return 1;
	}
}

/*
int myReadN(FT_HANDLE ftHandle,LPVOID lpBuffer,DWORD dwBytesToRead,LPDWORD lpBytesRead)
{
	DWORD dwTotal = 0;
	DWORD dwIn = 0;
	unsigned char* pNext = (unsigned char*)lpBuffer;
	
	int i=1;
	do{
		dwIn = 0;
		DWORD size = (dwBytesToRead-dwTotal<RX_SECTOR)?(dwBytesToRead-dwTotal):RX_SECTOR;
		FT_STATUS status = FT_Read(ftHandle,&pNext[dwTotal],size,&dwIn);
		if(status==FT_OK && dwIn>0){
			dwTotal += dwIn;
			//printf("FT_Read第%d次读取到%u个字节\n",i,dwIn);
		}else if(status!=FT_OK){
			//printf("FT_Read失败[错误码:%d]\n",status);
			break;
		}else{
			//printf("FT_Read失败[读取数据%u<=0]\n",dwIn);
			break;
		}
		i++;
		
	}while(dwTotal<dwBytesToRead);
	
	//FT_STATUS status = FT_Purge(ftHandle, FT_PURGE_TX|FT_PURGE_RX);
	//if(status!=FT_OK){
	//	printf("FT_Purge失败[错误码:%d]\n",status);
	//	return false;
	//}
	*lpBytesRead = dwTotal;
	if(dwTotal<dwBytesToRead){
		printf("ReadN failed [read %u<%u (%u lost)]\n",dwTotal,dwBytesToRead,dwBytesToRead-dwTotal);
		return 0;
	}else{
		printf("ReadN success. [read %u]\n",dwBytesToRead);
		return 1;
	}
}
*/

int myReadN(FT_HANDLE ftHandle,LPVOID lpBuffer,DWORD dwBytesToRead,LPDWORD lpBytesRead)
{
	DWORD dwTotal = 0;
	DWORD dwIn = 0;
	unsigned char* pNext = (unsigned char*)lpBuffer;
	DWORD dwInQueue = 0;
	
	do{	
		Sleep(100);
		FT_STATUS status = FT_GetQueueStatus(ftHandle, &dwInQueue);
		if (FT_OK==status) 
		{
			printf("FT_GetQueueStatus success. [%u in queue]\n",dwInQueue);
			dwIn = 0;
			if (dwInQueue>0) {
				
				status = FT_Read(ftHandle,&pNext[dwTotal],dwInQueue,&dwIn);
				if (status==FT_OK && dwIn>0) {
					dwTotal += dwIn;
					printf("FT_Read success. [read %u/%u]\n",dwIn,dwTotal);
				}else if(status!=FT_OK){
					printf("FT_Read failed. (error %d)\n",status);
					break;
				}else{
					printf("FT_Read failed. [read<0]\n");
					break;
				}
			}
			else
			{
				break;
			}
			
		}else{
			printf("FT_GetQueueStatus failed. (error %d)\n",status);
			break;
		}
		
	}while(dwTotal<dwBytesToRead);
	
	//FT_STATUS status = FT_Purge(ftHandle, FT_PURGE_TX|FT_PURGE_RX);
	//if(status!=FT_OK){
	//	printf("FT_Purge失败[错误码:%d]\n",status);
	//	return false;
	//}
	*lpBytesRead = dwTotal;
	if(dwTotal<dwBytesToRead){
		printf("ReadN failed [read %u<%u (%u lost)]\n",dwTotal,dwBytesToRead,dwBytesToRead-dwTotal);
		return 0;
	}else{
		printf("ReadN success. [read %u]\n",dwBytesToRead);
		return 1;
	}
}

void myClearRead(FT_HANDLE ftHandle)
{
	DWORD dwRxBytes=0,dwTxBytes=0,dwEventDWord=0;
	
	FT_STATUS status = FT_GetStatus(ftHandle,&dwRxBytes,&dwTxBytes,&dwEventDWord);
	if(status!=FT_OK){
		printf("FT_GetStatus() failed. (error %d)\n",status);
		return;
	}else{
		printf("FT_GetStatus(%u,%u,%u) success.\n",dwRxBytes,dwTxBytes,dwEventDWord);
		if(dwRxBytes>0){
			DWORD dwIn = 0;
			unsigned char* buf = new unsigned char[dwRxBytes];
			if(myReadN(ftHandle,buf,dwRxBytes,&dwIn)==1){
				printf("clear data success. [%u]\n",dwIn);
			}else{
				printf("clear data fail. [%u]\n",dwIn);
			}
			delete[] buf;
		}
	}
}

void fillWithUint2(void* data, unsigned short value, bool bigEndian)
{
	unsigned char* pSrc = (unsigned char*) &value;
	unsigned char* pDes = (unsigned char*) data;
	if(bigEndian){
		pDes[0]=pSrc[1];
		pDes[1]=pSrc[0];
	}else{
		pDes[0]=pSrc[0];
		pDes[1]=pSrc[1];
	}
}

void fillWithUint4(void* data, unsigned int value, bool bigEndian)
{
	unsigned char* pSrc = (unsigned char*) &value;
	unsigned char* pDes = (unsigned char*) data;
	if(bigEndian){
		pDes[0]=pSrc[3];
		pDes[1]=pSrc[2];
		pDes[2]=pSrc[1];
		pDes[3]=pSrc[0];
	}else{
		pDes[0]=pSrc[0];
		pDes[1]=pSrc[1];
		pDes[2]=pSrc[2];
		pDes[3]=pSrc[3];
	}
}

unsigned short toUint2(const void* data,bool bigEndian)
{
	unsigned short n = 0;
	unsigned char* pSrc = (unsigned char*) data;
	unsigned char* pDes = (unsigned char*) &n;
	if(bigEndian){
		pDes[0]=pSrc[1];
		pDes[1]=pSrc[0];
	}else{
		pDes[0]=pSrc[0];
		pDes[1]=pSrc[1];
	}
	return n;
}

int main()
{
	unsigned int n=0;
	printf("point number = ");
	scanf("%u",&n);

	FT_HANDLE ftHandle;
	FT_STATUS status = FT_OK;
	
	if(myOpen(0, &ftHandle)!=1){
		return -1;
	}
	
	status = mySetup(ftHandle);
	if(status!=FT_OK){
		return -1;
	}
	
	myClearRead(ftHandle);
	
	unsigned char cmdReadData[16] = {
		0xAA,0xAA,0x55,0x55,	0x40,0x02,	0x00,0x00,0x00,0x01,
		0x00,0x00,0x00,0x00,
		0x00,0x01
	};
	unsigned int length = n;
	unsigned int nBufLen = length*4;
	DWORD dwOut=0,dwIn=0;
	unsigned char* bufReadData = new unsigned char[nBufLen];
	
	// 填写读取数据指令的数据长度字段及校验和字段
	fillWithUint4(&cmdReadData[10],length,true);
	unsigned short chk = toUint2(&cmdReadData[10],true)+toUint2(&cmdReadData[12],true);
	fillWithUint2(&cmdReadData[14],chk,true);
	
	printf("Prepare to read %u\n",nBufLen);
	
	//for(int i=0;i<10;i++){
		if(myWrite(ftHandle,cmdReadData,16,&dwOut)==1){
			if(myReadN(ftHandle,bufReadData,length*4,&dwIn)==1){
				printf("read data success. [%u]\n",dwIn);
			}else{
				printf("read data fail. [%u]\n",dwIn);
			}
		}
	//}
	
	delete[] bufReadData;
	
	myClearRead(ftHandle);
	
	myClose(ftHandle);

	system("pause");
	
	return 0;
}




